' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE "SDL.INC"
#INCLUDE "GLU.INC"
#INCLUDE "GDIPUTILS.INC"

$WindowCaption = "NeHe Lesson 19"

%MAX_PARTICLES = 1000   ' Number of particles to create

TYPE particles
  active AS LONG    ' Active (Yes/No)
  life AS SINGLE    ' Particle life
  fade AS SINGLE    ' Fade speed
  r AS SINGLE       ' Red value
  g AS SINGLE       ' Green value
  b AS SINGLE       ' Blue value
  x AS SINGLE       ' X Position
  y AS SINGLE       ' Y Position
  z AS SINGLE       ' Z Position
  xi AS SINGLE      ' X Direction
  yi AS SINGLE      ' Y Direction
  zi AS SINGLE      ' Z Direction
  xg AS SINGLE      ' X Gravity
  yg AS SINGLE      ' Y Gravity
  zg AS SINGLE      ' Z Gravity
END TYPE

GLOBAL hDC AS LONG             ' Device context handle
GLOBAL TextureHandle AS DWORD  ' Texture handle
GLOBAL rainbow AS LONG         ' Rainbow mode?
GLOBAL slowdown AS SINGLE      ' Slow down particles
GLOBAL xspeed AS SINGLE        ' Base x speed (to allow keyboard direction of tail)
GLOBAL yspeed AS SINGLE        ' Base y speed (to allow keyboard direction of tail)
GLOBAL zoom AS SINGLE          ' Used to zoom out
GLOBAL clr AS DWORD            ' Current color selection
GLOBAL delay AS DWORD          ' Rainbow effec delay
GLOBAL sp AS LONG
GLOBAL particle() AS particles
GLOBAL colors() AS SINGLE

' =======================================================================================
' All the setup goes here
' =======================================================================================
SUB SetupScene ()

   LOCAL hr AS LONG
   LOCAL strTextureData AS STRING
   LOCAL TextureWidth, TextureHeight AS LONG
   LOCAL i AS LONG

   slowdown = 2.0!
   zoom = -40.0!

   DIM   particle (%MAX_PARTICLES - 1) AS particles   ' Particle array (room for particle info)
   DIM   colors (11, 2) AS SINGLE

   colors ( 0, 0) = 1.0!  : colors ( 0, 1) = 0.5!  : colors ( 0, 2) = 0.5!
   colors ( 1, 0) = 1.0!  : colors ( 1, 1) = 0.75! : colors ( 1, 2) = 0.5!
   colors ( 2, 0) = 1.0!  : colors ( 2, 1) = 1.0!  : colors ( 2, 2) = 0.5!
   colors ( 3, 0) = 0.75! : colors ( 3, 1) = 1.0!  : colors ( 3, 2) = 0.5!
   colors ( 4, 0) = 0.5!  : colors ( 4, 1) = 1.0!  : colors ( 4, 2) = 0.5!
   colors ( 5, 0) = 0.5!  : colors ( 5, 1) = 1.0!  : colors ( 5, 2) = 0.75!
   colors ( 6, 0) = 0.5!  : colors ( 6, 1) = 1.0!  : colors ( 6, 2) = 1.0!
   colors ( 7, 0) = 0.5!  : colors ( 7, 1) = 0.75! : colors ( 7, 2) = 1.0!
   colors ( 8, 0) = 0.5!  : colors ( 8, 1) = 0.5!  : colors ( 8, 2) = 1.0!
   colors ( 9, 0) = 0.75! : colors ( 9, 1) = 0.5!  : colors ( 9, 2) = 1.0!
   colors (10, 0) = 1.0!  : colors (10, 1) = 0.5!  : colors (10, 2) = 1.0!
   colors (11, 0) = 1.0!  : colors (11, 1) = 0.5! :  colors (11, 2) = 0.75!

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("particle.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   ' Assign handle
   glGenTextures 1, TextureHandle

   ' Create linear filtered texture
   glBindTexture %GL_TEXTURE_2D, TextureHandle
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Select smooth shading
   glShadeModel %GL_SMOOTH
   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Disable depth testing
   glDisable %GL_DEPTH_TEST
   ' Enable blending
   glEnable %GL_BLEND
   ' Type of blending to perform
   glBlendFunc %GL_SRC_ALPHA, %GL_ONE
	' Really nice perspective calculations
   glHint %GL_PERSPECTIVE_CORRECTION_HINT, %GL_NICEST
   ' Really nice point smoothing
   glHint %GL_POINT_SMOOTH_HINT, %GL_NICEST
   ' Enable texture maping
   glEnable %GL_TEXTURE_2D
   ' Select texture
   glBindTexture %GL_TEXTURE_2D, TextureHandle

   ' Initializes all the particles
   FOR i = 0 TO %MAX_PARTICLES -1
      particle(i).active = %True                             ' Make all the particles active
      particle(i).life = 1.0!                                ' Give all the particles full life
      particle(i).fade = (RND * 100) / 1000.0! + 0.003!  ' Random fade speed
      particle(i).r = colors(i * (12 \ %MAX_PARTICLES), 0)   ' Select red rainbow color
      particle(i).g = colors(i * (12 \ %MAX_PARTICLES), 1)   ' Select red rainbow color
      particle(i).b = colors(i * (12 \ %MAX_PARTICLES), 2)   ' Select red rainbow color
      particle(i).xi = ((RND * 50) - 26.0!) * 10.0!      ' Random speed on x axis
      particle(i).yi = ((RND * 50) - 25.0!) * 10.0!      ' Random speed on y axis
      particle(i).zi = ((RND * 50) - 25.0!) * 10.0!      ' Random speed on z axis
      particle(i).xg = 0.0!                                  ' Set horizontal pull to zero
      particle(i).yg = -0.8!                                 ' Set vertical pull downward
      particle(i).zg = 0.0!                                  ' Set pull on z axis to zero
   NEXT

END SUB
' =======================================================================================

' =======================================================================================
' Draw the scene
' =======================================================================================
SUB DrawScene (BYVAL nWidth AS LONG, BYVAL nHeight AS LONG)

   LOCAL i AS LONG
   LOCAL sx, sy, sz AS SINGLE

   ' Clear the screen buffer
   glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

   ' Select and setup the projection matrix
   glMatrixMode %GL_PROJECTION
   glLoadIdentity

   ' Prevent divide by zero making height equal one
   IF nHeight = 0 THEN nHeight = 1

   ' Calculate the aspect ratio of the window
   gluPerspective 45.0!, nWidth/nHeight, 1.0!, 100.0!

   ' Select and setup the modelview matrix
   glMatrixMode %GL_MODELVIEW
   glLoadIdentity

   ' Loop through all the particles
   FOR i = 0 TO %MAX_PARTICLES - 1
      IF particle(i).active THEN           ' If the particle is active
         sx = particle(i).x                ' Grab our particle x position
         sy = particle(i).y                ' Grab our particle y position
         sz = particle(i).z + zoom         ' Particle z pos + zoom

         ' draw the particle using our rgb values, fade the particle based on it's life
         glColor4f particle(i).r, particle(i).g, particle(i).b, particle(i).life
         glBegin %GL_TRIANGLE_STRIP                                ' build quad from a triangle strip
            glTexCoord2d 1, 1 : glVertex3f sx + 0.5!, sy + 0.5!, sz  ' Top right
            glTexCoord2d 0, 1 : glVertex3f sx - 0.5!, sy + 0.5!, sz  ' Top left
            glTexCoord2d 1, 0 : glVertex3f sx + 0.5!, sy - 0.5!, sz  ' Bottom right
            glTexCoord2d 0, 0 : glVertex3f sx - 0.5!, sy - 0.5!, sz  ' Bottom left
         glEnd                                                     ' Done building triangle strip

         particle(i).x = particle(i).x + particle(i).xi / (slowdown * 1000)  ' Move on the x axis by x speed
         particle(i).y = particle(i).y + particle(i).yi / (slowdown * 1000)  ' Move on the y axis by y speed
         particle(i).z = particle(i).z + particle(i).zi / (slowdown * 1000)  ' Move on the z axis by z speed

         particle(i).xi = particle(i).xi + particle(i).xg          ' Take pull on x axis into account
         particle(i).yi = particle(i).yi + particle(i).yg          ' Take pull on y axis into account
         particle(i).zi = particle(i).zi + particle(i).zg          ' Take pull on z axis into account
         particle(i).life = particle(i).life - particle(i).fade    ' Reduce particles life by 'fade'

         IF particle(i).life < 0.0! THEN                           ' If particle is burned out
            particle(i).life = 1.0!                                ' Give it new life
            particle(i).fade = (RND * 100) / 1000.0! + 0.003!  ' Random fade value
            particle(i).x = 0.0!                                   ' Center on x axis
            particle(i).y = 0.0!                                   ' Center on y axis
            particle(i).z = 0.0!                                   ' Center on z axis
            particle(i).xi = xspeed + ((RND * 60) - 32.0!)     ' X axis speed and direction
            particle(i).yi = yspeed + ((RND * 60) - 30.0!)     ' Y axis speed and direction
            particle(i).zi = ((RND * 60) - 30.0!)              ' Z axis speed and direction
            particle(i).r = colors(clr, 0)                         ' Select red from color table
            particle(i).g = colors(clr, 1)                         ' Select green from color table
            particle(i).b = colors(clr, 2)                         ' Select blue from color table
         END IF
      END IF
   NEXT

   IF (rainbow AND (delay > 25)) THEN
      sp = %TRUE                 ' Set flag telling us space is pressed
      delay = 0                  ' Reset the rainbow color cycling delay
      clr = clr + 1              ' Change the particle color
      IF clr > 11 THEN clr = 0   ' If color is too high, reset it
   END IF

   ' // Swap buffers
   SDL_GL_SwapBuffers

END SUB
' =======================================================================================

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL pscreen AS SDL_Surface PTR
   LOCAL nWidth, nHeight, bpp AS LONG
   LOCAL lResult, bFullScreen AS LONG
   LOCAL szCaption AS ASCIIZ * 256
   LOCAL dwFlags AS DWORD
   LOCAL t, t0, fps AS DOUBLE
   LOCAL nFrames AS LONG
   LOCAL rp AS LONG

   szCaption = $WindowCaption   ' Window caption

	' Ask the user which screen mode he prefers
	lResult = MessageBox(%NULL, "Would you like to run in fullscreen mode?", _
             "Start fullScreen?", %MB_YESNOCANCEL OR %MB_ICONQUESTION)
   SELECT CASE lResult
      CASE %IDCANCEL : EXIT FUNCTION
      CASE %IDYES    : bFullScreen = %TRUE
      CASE %IDNO     : bFullScreen = %FALSE
   END SELECT

   ' // Initialize SDL's subsystems - in this case, only video.
   IF SDL_Init(%SDL_INIT_VIDEO) < 0 THEN
      MSGBOX "Unable to init SDL: " & SDL_GetError()
      EXIT FUNCTION
   END IF

   ' // Attempt to create a 640x480 window
   nWidth = 640 : nHeight = 480 : bpp = 16
   IF bFullScreen THEN
      dwFlags = %SDL_OPENGL OR %SDL_FULLSCREEN
   ELSE
      dwFlags = %SDL_OPENGL
   END IF
   pscreen = SDL_SetVideoMode(nWidth, nHeight, bpp, dwFlags)
   SDL_WM_SetCaption szCaption, ""

   ' // If we fail, return error.
   IF pscreen = %NULL THEN
      SDL_Quit
      MSGBOX "Unable to set the video mode: " & SDL_GetError()
      EXIT FUNCTION
   END IF

   ' // Set up the scene
   SetupScene

   ' Enable keyboard repeat
   SDL_EnableKeyRepeat %SDL_DEFAULT_REPEAT_DELAY, %SDL_DEFAULT_REPEAT_INTERVAL

   ' // Main loop
   LOCAL done AS LONG
   LOCAL uevent AS SDL_Event
   WHILE done = %FALSE

      ' // Poll for events, and handle the ones we care about.
      WHILE SDL_PollEvent(VARPTR(uevent))
         SELECT CASE uevent.type
            CASE %SDL_KEYDOWN
               SELECT CASE uevent.key.keysym.sym
                  CASE %SDLK_MINUS
                     IF slowdown < 4.0! THEN slowdown = slowdown - 0.01!
                  CASE %SDLK_RETURN
                     IF ISFALSE rp THEN
                        rp = %TRUE
                        rainbow = NOT rainbow
                     END IF
                  ' Space or rainbow mode
                  CASE %SDLK_SPACE
                     IF (ISFALSE sp) OR (rainbow AND (delay > 25)) THEN
                        rainbow = %FALSE           ' Disable rainbow mode
                        sp = %TRUE                 ' Set flag telling us space is pressed
                        delay = 0                  ' Reset the rainbow color cycling delay
                        clr = clr + 1              ' Change the particle color
                        IF clr > 11 THEN clr = 0   ' If color is too high, reset it
                     END IF
                  CASE %SDLK_PAGEUP
                     zoom = zoom - 0.1!
                  CASE %SDLK_PAGEDOWN
                     zoom = zoom + 0.1!
                  CASE %SDLK_UP
                     IF yspeed < 200 THEN yspeed = yspeed + 1.0!
                  CASE %SDLK_DOWN
                     IF yspeed > - 200 THEN yspeed = yspeed - 1.0!
                  CASE %SDLK_RIGHT
                     IF xspeed < 200 THEN xspeed = xspeed + 1.0!
                  CASE %SDLK_LEFT
                     IF xspeed > -200 THEN xspeed = xspeed - 1.0!
               END SELECT
            CASE %SDL_KEYUP
               ' // Quit if escape is pressed
               IF uevent.key.keysym.sym = %SDLK_ESCAPE THEN
                  done = %TRUE
                  EXIT LOOP
               END IF
               ' Reset flags
               rp = %FALSE
               sp = %FALSE
            CASE %SDL_QUIT
               done = %TRUE
               EXIT LOOP
         END SELECT
      WEND

      IF ISFALSE bFullScreen THEN
         ' Get time and mouse position
         t = INT(TIMER)
         ' Calculate and display FPS (frames per second)
         IF t > t0 OR nFrames = 0 THEN
            fps = nFrames \ (t - t0)
            wsprintf szCaption, $WindowCaption & " (%i FPS)", BYVAL fps
            SDL_WM_SetCaption szCaption, szCaption
            t0 = t
            nFrames = 0
         END IF
         nFrames = nFrames + 1
      END IF

      ' // Draw the scene
      DrawScene nWidth, nHeight

   WEND

   ' Delete the texture
   IF TextureHandle THEN glDeleteTextures(1, TextureHandle)

   ' Shut down SDL
   SDL_Quit

END FUNCTION
' =======================================================================================
